@@ -63,10 +63,11 @@ in this Software without prior written authorization from The Open Group.
 #include	"difs.h"
 #include	"dispatch.h"
 #include	"swaprep.h"
+#include        <X11/fonts/libxfont2.h>
 
 static FontPathElementPtr *font_path_elements = (FontPathElementPtr *) 0;
 static int  num_fpes = 0;
-static FPEFunctions *fpe_functions = (FPEFunctions *) 0;
+static xfont2_fpe_funcs_rec *fpe_functions;
 static int  num_fpe_types = 0;
 
 static int  num_slept_fpes = 0;
@@ -107,8 +108,8 @@ FreeFPE(FontPathElementPtr fpe)
     fpe->refcount--;
     if (fpe->refcount == 0) {
 	(*fpe_functions[fpe->type].free_fpe) (fpe);
-	fsfree(fpe->name);
-	fsfree(fpe);
+	FSfree(fpe->name);
+	FSfree(fpe);
     }
 }
 
@@ -139,8 +140,8 @@ QueueFontWakeup(FontPathElementPtr fpe)
     }
     if (num_slept_fpes == size_slept_fpes) {
 	new = (FontPathElementPtr *)
-	    fsrealloc(slept_fpes,
-		      sizeof(FontPathElementPtr) * (size_slept_fpes + 4));
+	    FSreallocarray(slept_fpes, (size_slept_fpes + 4),
+			   sizeof(FontPathElementPtr));
 	if (!new)
 	    return;
 	slept_fpes = new;
@@ -179,7 +180,7 @@ FontWakeup(pointer data, int count, unsigned long *lastSelectMask)
     /* wake up any fpe's that may be waiting for information */
     for (i = 0; i < num_slept_fpes; i++) {
 	fpe = slept_fpes[i];
-	(void) (*fpe_functions[fpe->type].wakeup_fpe) (fpe, lastSelectMask);
+	(void) (*fpe_functions[fpe->type].wakeup_fpe) (fpe);
     }
 }
 
@@ -193,8 +194,9 @@ add_id_to_list(FontIDListPtr ids, Font fid)
      */
     if (ids->num == ids->size) {
 	/* increase size of array */
-	newlist = (Font *) fsrealloc(ids->client_list,
-			      sizeof(Font) * (ids->size + NUM_IDS_PER_CLIENT));
+	newlist = (Font *) FSreallocarray(ids->client_list,
+					  (ids->size + NUM_IDS_PER_CLIENT),
+					  sizeof(Font));
 	if (!newlist)
 	    return FALSE;
 	ids->client_list = newlist;
@@ -229,14 +231,13 @@ make_clients_id_list(void)
     FontIDListPtr ids;
     Font       *fids;
 
-    ids = (FontIDListPtr) fsalloc(sizeof(FontIDListRec));
-    fids = (Font *) fsalloc(sizeof(Font) * NUM_IDS_PER_CLIENT);
+    ids = (FontIDListPtr) FSalloc(sizeof(FontIDListRec));
+    fids = (Font *) FScalloc(NUM_IDS_PER_CLIENT, sizeof(Font));
     if (!ids || !fids) {
-	fsfree(ids);
-	fsfree(fids);
+	FSfree(ids);
+	FSfree(fids);
 	return (FontIDListPtr) 0;
     }
-    bzero((char *) fids, sizeof(Font) * NUM_IDS_PER_CLIENT);
     ids->client_list = fids;
     ids->size = NUM_IDS_PER_CLIENT;
     ids->num = 0;
@@ -254,11 +255,11 @@ free_svrPrivate(pointer svrPrivate)
 	for (i = 0; i < MAXCLIENTS; i++) {
 	    ids = idlist[i];
 	    if (ids) {
-		fsfree((char *) ids->client_list);
-		fsfree((char *) ids);
+		FSfree((char *) ids->client_list);
+		FSfree((char *) ids);
 	    }
 	}
-	fsfree((char *) idlist);
+	FSfree((char *) idlist);
     }
 }
 
@@ -301,7 +302,7 @@ do_open_font(ClientPtr client, pointer data)
 
 	if (err == FontNameAlias && alias) {
 	    newlen = strlen(alias);
-	    newname = (char *) fsrealloc(cPtr->fontname, newlen);
+	    newname = (char *) FSrealloc(cPtr->fontname, newlen);
 	    if (!newname) {
 		err = AllocError;
 		break;
@@ -333,7 +334,7 @@ do_open_font(ClientPtr client, pointer data)
 	err = BadFontName;
 	goto dropout;
     }
-    cfp = (ClientFontPtr) fsalloc(sizeof(ClientFontRec));
+    cfp = (ClientFontPtr) FSalloc(sizeof(ClientFontRec));
     if (!cfp) {
 	err = AllocError;
 	goto dropout;
@@ -342,7 +343,8 @@ do_open_font(ClientPtr client, pointer data)
     cfp->clientindex = cPtr->client->index;
 
     if (fontPatternCache && pfont != cPtr->non_cachable_font)
-	CacheFontPattern(fontPatternCache, cPtr->orig_name, cPtr->orig_len, pfont);
+	xfont2_cache_font_pattern(fontPatternCache, cPtr->orig_name,
+                                  cPtr->orig_len, pfont);
 
     /* either pull out the other id or make the array */
     if (pfont->refcnt != 0) {
@@ -352,33 +354,33 @@ do_open_font(ClientPtr client, pointer data)
 	    ids = make_clients_id_list();
 	    if (!ids) {
 		err = AllocError;
-		fsfree(cfp);
+		FSfree(cfp);
 		goto dropout;
 	    }
 	    idlist[cPtr->client->index] = ids;
 	}
 	orig = (ids->num > 0) ? ids->client_list[0] : (Font)0;
     } else {
-	idlist = (FontIDListPtr *) fsalloc(sizeof(FontIDListPtr) * MAXCLIENTS);
+	idlist = (FontIDListPtr *)
+            FScalloc(MAXCLIENTS, sizeof(FontIDListPtr));
 	if (!idlist) {
 	    err = AllocError;
-	    fsfree(cfp);
+	    FSfree(cfp);
 	    goto dropout;
 	}
 	ids = make_clients_id_list();
 	if (!ids) {
 	    err = AllocError;
-	    fsfree(idlist);
-	    fsfree(cfp);
+	    FSfree(idlist);
+	    FSfree(cfp);
 	    goto dropout;
 	}
-	bzero((char *) idlist, (sizeof(FontIDListPtr) * MAXCLIENTS));
 	idlist[cPtr->client->index] = ids;
 	orig = (Font) 0;
 	pfont->svrPrivate = (pointer) idlist;
     }
     if (!AddResource(cPtr->client->index, cPtr->fontid, RT_FONT, (pointer) cfp)) {
-	fsfree(cfp);
+	FSfree(cfp);
 	free_svrPrivate(pfont->svrPrivate);
 	pfont->svrPrivate = (pointer) 0;
 	err = AllocError;
@@ -413,9 +415,9 @@ dropout:
     for (i = 0; i < cPtr->num_fpes; i++) {
 	FreeFPE(cPtr->fpe_list[i]);
     }
-    fsfree(cPtr->fpe_list);
-    fsfree(cPtr->fontname);
-    fsfree(cPtr);
+    FSfree(cPtr->fpe_list);
+    FSfree(cPtr->fontname);
+    FSfree(cPtr);
     return TRUE;
 }
 
@@ -459,7 +461,8 @@ OpenFont(
     */
 
     if (fontPatternCache &&
-	  (pfont = FindCachedFontPattern(fontPatternCache, name, namelen)) &&
+	  (pfont = xfont2_find_cached_font_pattern(fontPatternCache, name,
+                                                   namelen)) &&
 	   pfont->info.cachable) {
 	ClientFontPtr cfp;
 
@@ -472,7 +475,7 @@ OpenFont(
 	    }
 	    idlist[client->index] = ids;
 	}
-	cfp = (ClientFontPtr) fsalloc(sizeof(ClientFontRec));
+	cfp = (ClientFontPtr) FSalloc(sizeof(ClientFontRec));
 	if (!cfp) {
     lowmem:
 	    SendErrToClient(client, FSBadAlloc, (pointer) 0);
@@ -501,12 +504,12 @@ OpenFont(
 	    return FSSuccess;
 	}
     }
-    c = (OFclosurePtr) fsalloc(sizeof(OFclosureRec));
+    c = (OFclosurePtr) FSalloc(sizeof(OFclosureRec));
     if (!c)
 	goto lowmem;
-    c->fontname = (char *) fsalloc(namelen);
+    c->fontname = (char *) FSalloc(namelen);
     if (!c->fontname) {
-	fsfree(c);
+	FSfree(c);
 	goto lowmem;
     }
     /*
@@ -514,13 +517,13 @@ OpenFont(
      * while we're blocking, the request still appears atomic
      */
     c->fpe_list = (FontPathElementPtr *)
-	fsalloc(sizeof(FontPathElementPtr) * num_fpes);
+	FSallocarray(num_fpes, sizeof(FontPathElementPtr));
     if (!c->fpe_list) {
-	fsfree(c->fontname);
-	fsfree(c);
+	FSfree(c->fontname);
+	FSfree(c);
 	goto lowmem;
     }
-    memmove( c->fontname, name, namelen);
+    memcpy(c->fontname, name, namelen);
     for (i = 0; i < num_fpes; i++) {
 	c->fpe_list[i] = font_path_elements[i];
 	UseFPE(c->fpe_list[i]);
@@ -550,7 +553,7 @@ close_font(FontPtr pfont)
     assert(pfont);
     if (--pfont->refcnt == 0) {
 	if (fontPatternCache)
-	    RemoveCachedFontPattern(fontPatternCache, pfont);
+	    xfont2_remove_cached_font_pattern(fontPatternCache, pfont);
 	fpe = pfont->fpe;
 	free_svrPrivate(pfont->svrPrivate);
 	(*fpe_functions[fpe->type].close_font) (fpe, pfont);
@@ -574,7 +577,7 @@ CloseClientFont(
     ids = idlist[cfp->clientindex];
     remove_id_from_list(ids, fid);
     ret = close_font(cfp->font);
-    fsfree((char *) cfp);
+    FSfree((char *) cfp);
     return ret;
 }
 
@@ -601,7 +604,7 @@ free_font_path(FontPathElementPtr *list, int n)
     for (i = 0; i < n; i++) {
 	FreeFPE(list[i]);
     }
-    fsfree((char *) list);
+    FSfree((char *) list);
 }
 
 static FontPathElementPtr
@@ -641,7 +644,7 @@ set_font_path_elements(
     FontPathElementPtr fpe, *fplist;
 
     fplist = (FontPathElementPtr *)
-	fsalloc(sizeof(FontPathElementPtr) * npaths);
+	FSallocarray(npaths, sizeof(FontPathElementPtr));
     if (!fplist) {
 	*bad = 0;
 	return FSBadAlloc;
@@ -670,7 +673,7 @@ set_font_path_elements(
 		}
 		/* can't do it, so act like it's a new one */
 	    }
-	    name = (char *) fsalloc(len + 1);
+	    name = (char *) FSalloc(len + 1);
 	    if (!name) {
 		err = FSBadAlloc;
 		goto bail;
@@ -681,14 +684,14 @@ set_font_path_elements(
 	    if (type == -1)
 	    {
 		NoticeF("ignoring font path element %s (bad font path descriptor)\n", name);
-		fsfree(name);
+		FSfree(name);
 		cp += len;
 		continue;
 	    }
 	    /* must be new -- make it */
-	    fpe = (FontPathElementPtr) fsalloc(sizeof(FontPathElementRec));
+	    fpe = (FontPathElementPtr) FSalloc(sizeof(FontPathElementRec));
 	    if (!fpe) {
-		fsfree(name);
+		FSfree(name);
 		err = FSBadAlloc;
 		goto bail;
 	    }
@@ -701,8 +704,8 @@ set_font_path_elements(
 	    err = (*fpe_functions[fpe->type].init_fpe) (fpe);
 	    if (err != Successful) {
 		NoticeF("ignoring font path element %s (unreadable)\n", fpe->name);
-		fsfree(fpe->name);
-		fsfree(fpe);
+		FSfree(fpe->name);
+		FSfree(fpe);
 		continue;
 	    }
 	    fplist[validpaths++] = fpe;
@@ -710,7 +713,7 @@ set_font_path_elements(
     }
     if (validpaths < npaths) {
 	FontPathElementPtr *ftmp = (FontPathElementPtr *)
-	    fsrealloc(fplist, sizeof(FontPathElementPtr) * validpaths);
+	    FSreallocarray(fplist, validpaths, sizeof(FontPathElementPtr));
 
 	if (!ftmp && validpaths)
 	    goto bail;
@@ -726,18 +729,18 @@ set_font_path_elements(
     font_path_elements = fplist;
     num_fpes = npaths;
     if (fontPatternCache)
-	EmptyFontPatternCache(fontPatternCache);
+	xfont2_empty_font_pattern_cache(fontPatternCache);
     return FSSuccess;
 bail:
     *bad = validpaths;
     while (--validpaths >= 0)
 	FreeFPE(fplist[i]);
-    fsfree(fplist);
+    FSfree(fplist);
     return err;
 }
 
 /*
- * expects comma seperated string
+ * expects comma separated string
  */
 int
 SetFontCatalogue(
@@ -754,6 +757,8 @@ SetFontCatalogue(
     len = strlen(str) + 1;
     paths = p = (char *) ALLOCATE_LOCAL(len);
     npaths = 0;
+    if (p == NULL)
+        return FSBadAlloc;
 
     while (*str) {
 	end = index(str, ',');
@@ -761,7 +766,7 @@ SetFontCatalogue(
 	    end = str + strlen(str);
 	}
 	*p++ = len = end - str;
-	memmove( p, str, len);
+	memcpy(p, str, len);
 	npaths++;
 	str += len;		/* skip entry */
 	if (*str == ',')
@@ -877,11 +882,11 @@ do_list_fonts_and_aliases(ClientPtr client, pointer data)
 		if (cPtr->haveSaved)
 		{
 		    if (cPtr->savedName)
-			(void)AddFontNamesName(cPtr->names, cPtr->savedName,
-					       cPtr->savedNameLen);
+			xfont2_add_font_names_name(cPtr->names, cPtr->savedName,
+                                                   cPtr->savedNameLen);
 		}
 		else
-		    (void)AddFontNamesName(cPtr->names, name, namelen);
+		    xfont2_add_font_names_name(cPtr->names, name, namelen);
 	    }
 
 	    /*
@@ -899,7 +904,7 @@ do_list_fonts_and_aliases(ClientPtr client, pointer data)
 		 * is BadFontName, indicating the alias resolution
 		 * is complete.
 		 */
-		memmove(tmp_pattern, resolved, resolvedlen);
+		memcpy(tmp_pattern, resolved, resolvedlen);
 		if (cPtr->haveSaved)
 		{
 		    char    *tmpname;
@@ -920,10 +925,10 @@ do_list_fonts_and_aliases(ClientPtr client, pointer data)
 		    cPtr->saved = cPtr->current;
 		    cPtr->haveSaved = TRUE;
 		    if (cPtr->savedName)
-			fsfree(cPtr->savedName);
-		    cPtr->savedName = (char *)fsalloc(namelen + 1);
+			FSfree(cPtr->savedName);
+		    cPtr->savedName = (char *)FSalloc(namelen + 1);
 		    if (cPtr->savedName)
-			memmove(cPtr->savedName, name, namelen + 1);
+			memcpy(cPtr->savedName, name, namelen + 1);
 		    cPtr->savedNameLen = namelen;
 		    aliascount = 20;
 		}
@@ -1030,10 +1035,10 @@ bail:
 	ClientWakeup(client);
     for (i = 0; i < cPtr->num_fpes; i++)
 	FreeFPE(cPtr->fpe_list[i]);
-    fsfree(cPtr->fpe_list);
-    if (cPtr->savedName) fsfree(cPtr->savedName);
-    FreeFontNames(names);
-    fsfree(cPtr);
+    FSfree(cPtr->fpe_list);
+    if (cPtr->savedName) FSfree(cPtr->savedName);
+    xfont2_free_font_names(names);
+    FSfree(cPtr);
     return TRUE;
 }
 
@@ -1058,19 +1063,19 @@ ListFonts(
 	return TRUE;
     }
 
-    if (!(c = (LFclosurePtr) fsalloc(sizeof *c)))
+    if (!(c = (LFclosurePtr) FSalloc(sizeof *c)))
 	goto badAlloc;
     c->fpe_list = (FontPathElementPtr *)
-	fsalloc(sizeof(FontPathElementPtr) * num_fpes);
+	FSallocarray(num_fpes, sizeof(FontPathElementPtr));
     if (!c->fpe_list) {
-	fsfree(c);
+	FSfree(c);
 	goto badAlloc;
     }
-    c->names = MakeFontNamesRecord(maxNames < 100 ? maxNames : 100);
+    c->names = xfont2_make_font_names_record(maxNames < 100 ? maxNames : 100);
     if (!c->names)
     {
-	fsfree(c->fpe_list);
-	fsfree(c);
+	FSfree(c->fpe_list);
+	FSfree(c);
 	goto badAlloc;
     }
     memmove( c->current.pattern, pattern, length);
@@ -1196,10 +1201,10 @@ do_list_fonts_with_info(ClientPtr client, pointer data)
 		cPtr->haveSaved = TRUE;
 		cPtr->savedNumFonts = numFonts;
 		if (cPtr->savedName)
-		  fsfree(cPtr->savedName);
-		cPtr->savedName = (char *)fsalloc(namelen + 1);
+		  FSfree(cPtr->savedName);
+		cPtr->savedName = (char *)FSalloc(namelen + 1);
 		if (cPtr->savedName)
-		  memmove(cPtr->savedName, name, namelen + 1);
+		  memcpy(cPtr->savedName, name, namelen + 1);
 		aliascount = 20;
 	    }
 	    memmove(cPtr->current.pattern, name, namelen);
@@ -1234,7 +1239,7 @@ do_list_fonts_with_info(ClientPtr client, pointer data)
 	    length = sizeof(*reply) + pFontInfo->nprops * sizeof(xFontProp);
 	    reply = cPtr->reply;
 	    if (cPtr->length < length) {
-		reply = (fsListFontsWithXInfoReply *) fsrealloc(cPtr->reply, length);
+		reply = (fsListFontsWithXInfoReply *) FSrealloc(cPtr->reply, length);
 		if (!reply) {
 		    err = AllocError;
 		    break;
@@ -1277,10 +1282,10 @@ do_list_fonts_with_info(ClientPtr client, pointer data)
 		(void) WriteToClient(client, lenpropdata, (char *) prop_info);
 	    }
 	    if (pFontInfo == &fontInfo) {
-		fsfree(fontInfo.props);
-		fsfree(fontInfo.isStringProp);
+		FSfree(fontInfo.props);
+		FSfree(fontInfo.isStringProp);
 	    }
-	    fsfree(prop_info);
+	    FSfree(prop_info);
 
 	    --cPtr->current.max_names;
 	    if (cPtr->current.max_names < 0)
@@ -1300,7 +1305,7 @@ do_list_fonts_with_info(ClientPtr client, pointer data)
 	else
 	    length = SIZEOF(fsListFontsWithXInfoReply);
 	if (cPtr->length < length) {
-	    final_reply = (fsGenericReply *) fsrealloc(cPtr->reply, length);
+	    final_reply = (fsGenericReply *) FSrealloc(cPtr->reply, length);
 	    if (final_reply) {
 		cPtr->reply = (fsListFontsWithXInfoReply *)final_reply;
 		cPtr->length = length;
@@ -1322,10 +1327,10 @@ bail:
 	ClientWakeup(client);
     for (i = 0; i < cPtr->num_fpes; i++)
 	FreeFPE(cPtr->fpe_list[i]);
-    fsfree(cPtr->fpe_list);
-    if (cPtr->savedName) fsfree(cPtr->savedName);
-    fsfree(cPtr->reply);
-    fsfree(cPtr);
+    FSfree(cPtr->fpe_list);
+    if (cPtr->savedName) FSfree(cPtr->savedName);
+    FSfree(cPtr->reply);
+    FSfree(cPtr);
     return TRUE;
 }
 
@@ -1350,12 +1355,12 @@ StartListFontsWithInfo(
 	return TRUE;
     }
 
-    if (!(c = (LFWXIclosurePtr) fsalloc(sizeof *c)))
+    if (!(c = (LFWXIclosurePtr) FSalloc(sizeof *c)))
 	goto badAlloc;
     c->fpe_list = (FontPathElementPtr *)
-	fsalloc(sizeof(FontPathElementPtr) * num_fpes);
+	FSallocarray(num_fpes, sizeof(FontPathElementPtr));
     if (!c->fpe_list) {
-	fsfree(c);
+	FSfree(c);
 	goto badAlloc;
     }
     memmove( c->current.pattern, pattern, length);
@@ -1403,52 +1408,17 @@ LoadGlyphRanges(
 
 
 int
-RegisterFPEFunctions(
-    NameCheckFunc name_func,
-    InitFpeFunc   init_func,
-    FreeFpeFunc   free_func,
-    ResetFpeFunc  reset_func,
-    OpenFontFunc  open_func,
-    CloseFontFunc close_func,
-    ListFontsFunc list_func,
-    StartLfwiFunc start_lfwi_func,
-    NextLfwiFunc  next_lfwi_func,
-    WakeupFpeFunc wakeup_func,
-    ClientDiedFunc client_died,
-    LoadGlyphsFunc load_glyphs,
-    StartLaFunc   start_list_alias_func,
-    NextLaFunc    next_list_alias_func,
-    SetPathFunc   set_path_func)
+register_fpe_funcs(const xfont2_fpe_funcs_rec *funcs)
 {
-    FPEFunctions *new;
+    xfont2_fpe_funcs_rec *new;
 
     /* grow the list */
-    new = (FPEFunctions *) fsrealloc(fpe_functions,
-				 (num_fpe_types + 1) * sizeof(FPEFunctions));
+    new = FSreallocarray(fpe_functions, (num_fpe_types + 1), sizeof(*new));
     if (!new)
 	return -1;
     fpe_functions = new;
 
-    fpe_functions[num_fpe_types].name_check = name_func;
-    fpe_functions[num_fpe_types].open_font = open_func;
-    fpe_functions[num_fpe_types].close_font = close_func;
-    fpe_functions[num_fpe_types].wakeup_fpe = wakeup_func;
-    fpe_functions[num_fpe_types].list_fonts = list_func;
-    fpe_functions[num_fpe_types].start_list_fonts_with_info =
-	start_lfwi_func;
-    fpe_functions[num_fpe_types].list_next_font_with_info =
-	next_lfwi_func;
-    fpe_functions[num_fpe_types].init_fpe = init_func;
-    fpe_functions[num_fpe_types].free_fpe = free_func;
-    fpe_functions[num_fpe_types].reset_fpe = reset_func;
-
-    fpe_functions[num_fpe_types].client_died = client_died;
-    fpe_functions[num_fpe_types].load_glyphs = load_glyphs;
-    fpe_functions[num_fpe_types].start_list_fonts_and_aliases =
-	start_list_alias_func;
-    fpe_functions[num_fpe_types].list_next_font_or_alias =
-	next_list_alias_func;
-    fpe_functions[num_fpe_types].set_path_hook = set_path_func;
+    memcpy(&fpe_functions[num_fpe_types], funcs, sizeof(*funcs));
 
     return num_fpe_types++;
 }
@@ -1486,9 +1456,9 @@ static int  fs_handlers_installed = 0;
 static unsigned int last_server_gen;
 
 int
-init_fs_handlers(
+xfs_init_fs_handlers(
     FontPathElementPtr fpe,
-    BlockHandlerProcPtr block_handler)
+    FontBlockHandlerProcPtr block_handler)
 {
     /* if server has reset, make sure the b&w handlers are reinstalled */
     if (last_server_gen < serverGeneration) {
@@ -1511,9 +1481,9 @@ init_fs_handlers(
 }
 
 void
-remove_fs_handlers(
+xfs_remove_fs_handlers(
     FontPathElementPtr fpe,
-    BlockHandlerProcPtr block_handler,
+    FontBlockHandlerProcPtr block_handler,
     Bool        all)
 {
     if (all) {
